/*-
 *
 * Copyright 2004-2008 Bruce Cran <bruce@cran.org.uk>. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <limits.h>
#include <unistd.h>
#include <getopt.h>
#include <errno.h>
#include <sys/types.h>

#include "mi/atadefs.h"
#include "mi/util.h"
#include "mi/atagen.h"

#ifdef __FreeBSD__
	#include <osreldate.h>

	#if __FreeBSD_version < 600034
		#error ATAidle requires FreeBSD 6.1 or newer
	#endif
#endif

int main( int argc, char ** argv )
{
    int rc = 0;
    ATA *ata;
    long opt_val;
    int ch;
    bool needsdevice;
    const char * const optstr = "hA:S:sI:iP:o";

    ata = malloc(sizeof(ATA));

    if (ata == 0)
    {
        /* malloc failed, abort */
        fprintf(stderr, "malloc failed, aborting.\n");
        exit(EXIT_FAILURE);
    }

    if( (argc == 1) || (!checkargs(argc, argv, optstr, &needsdevice)) )
        usage();

    if (geteuid() != 0)
    {
        printf("error: ATAidle requires root privileges to run\n");
        exit(EXIT_FAILURE);
    }

    if (needsdevice)
    {
        rc = ata_open(ata, argv[argc-1]);

        if (rc != 0)
        {
 		    printf("error opening device\n");
 		    exit(EXIT_FAILURE);
        }
    }

	/* now we've done all the checking of parameters etc.,
	 * let's see what the user wants us to do.
	 */
	if(!rc)
	{
		optind = 1;
		opterr = 1;

		while ((ch = getopt(argc, argv, optstr)) != -1)
		{
			switch(ch)
			{
				/* S = Standby */
				case 'S':
					rc = ata_strtolong( optarg, &opt_val );
					if(rc)
						printf("invalid standby value\n");
					else
						rc = ata_setstandbytimer( ata, opt_val );
					break;

				case 's':
			      rc = ata_setstandbytimer( ata, ATA_IDLEVAL_IMMEDIATE );
					break;

				/* o = Sleep (off) */
				case 'o':
					rc = ata_sleep( ata );
					break;

				/* I = Idle */
				case 'I':
					rc = ata_strtolong( optarg, &opt_val );

					if(rc)
						printf("invalid idle value\n");
					else
						rc = ata_setidletimer( ata, opt_val );
					break;

				case 'i':
					rc = ata_setidletimer(ata, ATA_IDLEVAL_IMMEDIATE);
					break;

				/* A = AutoAcoustic */
				case 'A':
					rc = ata_strtolong(optarg, &opt_val);
					if(rc)
						printf("invalid acoustic value\n");
					else
						rc = ata_setacoustic( ata, opt_val );
					break;

				/* P = Power (APM) */
				case 'P':
					rc = ata_strtolong(optarg, &opt_val);
					if(rc)
						printf("invalid apm value\n");
					else
						rc = ata_setapm( ata, opt_val );
					break;

				/* h = help */
				case 'h':
					usage();
					break;
			}
		}
	}

	/* fall-through: check if we've just got 1 arguments,
	 * the device: if so, just show information
	 * about that device
	 */
	if( argc == 2 && !rc )
	{
		printf("Device Info:\n\n");
		ata_showdeviceinfo( ata );
	}

	if (ata != NULL)
	{
		ata_close(ata);
		free(ata);
	}
	
	return rc;
}
